namespace :db do
  namespace :raw do
    desc 'Dump database to FILE or name of Rails.env'
    task :dump do
      verbose(true) unless Rake.application.options.silent

      struct = database_settings
      target = ENV['FILE'] || "#{Rails.env}.sql"
      target_tmp = "#{target}.tmp"
      adapter = struct.adapter

      case adapter
      when 'sqlite3'
        source = struct.database
        sh "sqlite3 #{Shellwords.shellescape source} .dump > #{Shellwords.shellescape target}"
      when 'postgresql'
        check_postgresql_host struct.host
        t_path_to_pg_dump = path_to 'pg_dump'
        sh "su - pe-postgres -c \'#{t_path_to_pg_dump} #{postgresql_credentials_for struct} --clean\' -s /bin/bash > #{Shellwords.shellescape target_tmp}"
        mv target_tmp, target
      else
        raise ArgumentError, "Unknown database adapter: #{adapter}"
      end
    end

    desc 'Restore database from FILE'
    task :restore do
      verbose(true) unless Rake.application.options.silent

      source = ENV['FILE']
      raise ArgumentError, 'No FILE argument specified to restore from' unless source

      struct = database_settings
      adapter = struct.adapter

      case adapter
      when 'sqlite3'
        target = struct.database
        mv target, "#{target}.old" if File.exist?(target)
        sh "sqlite3 #{Shellwords.shellescape target} < #{Shellwords.shellescape source}"
      when 'postgresql'
        check_postgresql_host struct.host
        t_path_to_psql = path_to 'psql'
        sh "su - pe-postgres -c \'#{t_path_to_psql} #{postgresql_credentials_for struct}\' -s /bin/bash < #{Shellwords.shellescape source}"
      else
        raise ArgumentError, "Unknown database adapter: #{adapter}"
      end

      Rake::Task['db:migrate'].invoke
    end

    desc "Optimize database tables to speed up queries. WARNING: May take a while."
    task :optimize, [:mode] => :environment do |task, args|
      case ActiveRecord::Base.connection.class.name
      when 'ActiveRecord::ConnectionAdapters::PostgreSQLAdapter'
        case args.mode
        when nil, 'reindex+vacuum'
          reindex = true
          vacuum = true
        when 'reindex'
          reindex = true
          vacuum = false
        when 'vacuum'
          reindex = false
          vacuum = true
        else
          raise "Invalid argument: '#{args.mode}', use one of 'reindex', 'vacuum', 'reindex+vacuum' or no argument"
        end

        if reindex then
          puts 'Rebuilding all indexes, this may take a while:'
          current_database = ActiveRecord::Base.connection.select_value('SELECT current_database()')
          ActiveRecord::Base.connection.execute("REINDEX DATABASE #{current_database}")
        end

        if vacuum then
          puts 'Optimizing table space, this may take a while:'
          ActiveRecord::Base.connection.execute('VACUUM FULL')
        end
      else
        raise "Don't know how to optimize for database engine: #{ActiveRecord::Base.connection.adapter_name}"
      end
    end
  end

  # Get full path to cmd
  def path_to(cmd)
    paths = ENV['PATH'].split(File::PATH_SEPARATOR)
    paths.insert(0, '/opt/puppet/bin')

    paths.each do |path|
      full_path_cmd = File.join(path, "#{cmd}")
      return full_path_cmd if File.exists? full_path_cmd
    end

    raise "Cannot find \'#{cmd}\'"
  end

  # In PostgreSQL only 'pe-postgres' user has the right privileges to restore a database. Currently, it's possible to login as 'pe-postgres' only from localhost.
  def check_postgresql_host(host)
    raise "Supported only for localhost" unless host.nil? || host == 'localhost'
  end

  # Return OpenStruct representing current environment's database.yml file.
  def database_settings
    require 'erb'
    require 'yaml'
    require 'ostruct'

    return @database_settings_cache ||= OpenStruct.new(
      YAML.load(
        ERB.new(
          File.read(
            File.join(Rails.root, 'config', 'database.yml'))).result)[Rails.env])
  end

  # Return string with PostgreSQL credentials for use on a command-line.
  def postgresql_credentials_for(struct)
    result = []
    result << "-p #{Shellwords.shellescape struct.port.to_s}" if struct.port
    result << "#{Shellwords.shellescape struct.database}"
    return result.join(' ')
  end
end
